function [map,z_low_lim] = adaptive_area_preserving_map_closed(v,f,lambda,bottom_coord)
% Compute an adaptive area-preserving map of a genus-0 closed surface to a 
% spherical cap with optimized conformality 
%
% Input:
% v: nv x 3 vertex coordinates of a genus-0 closed triangle mesh
% f: nf x 3 triangulations of a genus-0 closed triangle mesh
% lambda: balancing parameter for initial map (optional, default = 0.2)
% bottom_coord: 1 x 3 coordinates of the bottom center of the mesh (optional)
%
% Output:
% map: nv x 3 vertex coordinates of the closed adaptive area-preserving parameterization
% z_low_lim: the lower limit of the z-value of the sphere (everything is with z >= z_low_lim)
%
% If you use this code in your own work, please cite the following paper:
% G. P. T. Choi, A. Giri, L. Kumar, 
% "Adaptive area-preserving parameterization of open and closed anatomical surfaces."
% Computers in Biology and Medicine, 148, 105715, 2022.
% 
% Copyright (c) 2021-2022, Gary Pui-Tung Choi
% https://math.mit.edu/~ptchoi

%% Step 1: Puncturing a pair of triangles and perform initial flattening
fprintf('Step 1: Puncturing a pair of triangles and perform initial flattening...\n');
e = [f(:,[1 2]); f(:,[2 3]); f(:, [3 1])];
if nargin < 4
    % Remark: If the third input is not provided, we will assume that the 
    % surface is properly aligned with the z-axis in searching for a pair 
    % of triangles at the bottom part.
    % In case the surface is not properly aligned, use the optional input.
    [value,e_id] = sort((v(e(:,1),3) + v(e(:,2),3))*0.01 + ...
        abs(v(e(:,1),1) - mean(v(:,1))) + abs(v(e(:,2),1) - mean(v(:,1))) + ...
        abs(v(e(:,1),2) - mean(v(:,2))) + abs(v(e(:,2),2) - mean(v(:,2)))); % bottomost edge
else
    % sort the edges according to the distance between the edge center and
    % the given coordinates of the bottom center
    [value,e_id] = sort(sqrt(...
        ((v(e(:,1),1)+v(e(:,2),1))/2 - bottom_coord(1)).^2 + ...
        ((v(e(:,1),2)+v(e(:,2),2))/2 - bottom_coord(2)).^2 + ...
        ((v(e(:,1),3)+v(e(:,2),3))/2 - bottom_coord(3)).^2)); % bottomost edge
end
i = 1;
fid1 = mod(e_id(i)-1, length(f))+1;
fid2 = setdiff(find(sum(f==e(e_id(i),1) | f == e(e_id(i),2), 2) == 2), fid1);
diagonal1 = intersect(f(fid1,:), f(fid2,:));
diagonal2 = setdiff([f(fid1,:), f(fid2,:)], diagonal1);
lengthd1 = sqrt(sum((v(diagonal1(1),:) - v(diagonal1(2),:)).^2,2));
lengthd2 = sqrt(sum((v(diagonal2(1),:) - v(diagonal2(2),:)).^2,2));

edge1 = [diagonal1(1), diagonal2(1)];
edge2 = [diagonal1(1), diagonal2(2)];
edge3 = [diagonal1(2), diagonal2(1)];
edge4 = [diagonal1(2), diagonal2(2)];

length1 = sqrt(sum((v(edge1(1),:) - v(edge1(2),:)).^2,2));
length2 = sqrt(sum((v(edge2(1),:) - v(edge2(2),:)).^2,2));
length3 = sqrt(sum((v(edge3(1),:) - v(edge3(2),:)).^2,2));
length4 = sqrt(sum((v(edge4(1),:) - v(edge4(2),:)).^2,2));

fid1_initial = fid1;
fid2_initial = fid2;

try
    while lengthd1/lengthd2 > 1.2 || lengthd2/lengthd1 > 1.2 || ... % diagonals differ too much
            max([length1, length2, length3, length4])/min([length1, length2, length3, length4]) > 1.5
        i = i+1;
        fid1 = mod(e_id(i)-1, length(f))+1;
        fid2 = setdiff(find(sum(f==e(e_id(i),1) | f == e(e_id(i),2), 2) == 2), fid1);
        diagonal1 = intersect(f(fid1,:), f(fid2,:));
        diagonal2 = setdiff([f(fid1,:), f(fid2,:)], diagonal1);
        lengthd1 = sqrt(sum((v(diagonal1(1),:) - v(diagonal1(2),:)).^2,2));
        lengthd2 = sqrt(sum((v(diagonal2(1),:) - v(diagonal2(2),:)).^2,2));

        edge1 = [diagonal1(1), diagonal2(1)];
        edge2 = [diagonal1(1), diagonal2(2)];
        edge3 = [diagonal1(2), diagonal2(1)];
        edge4 = [diagonal1(2), diagonal2(2)];

        length1 = sqrt(sum((v(edge1(1),:) - v(edge1(2),:)).^2,2));
        length2 = sqrt(sum((v(edge2(1),:) - v(edge2(2),:)).^2,2));
        length3 = sqrt(sum((v(edge3(1),:) - v(edge3(2),:)).^2,2));
        length4 = sqrt(sum((v(edge4(1),:) - v(edge4(2),:)).^2,2));
    end
catch
    % none of the triangle pairs satisfy the required ratio, just use the
    % first pair
    fid1 = fid1_initial;
    fid2 = fid2_initial;
end
f([fid1,fid2],:) = [];

% Initial flattening map
% based on the stretch energy minimization (Yueh et al., J. Sci. Comput. 2017)
% and quasi-conformal composition (Choi et al., SIAM J. Imaging Sci. 2015)
uv = disk_conformal_map(v,f);
uv_SEM = Diskmap_SEM(f,v);
bd = meshboundaries(f);
uv_bdy = uv(bd{1},:);

% compute the Beltrami coefficient of the SEM map w.r.t. uv
mu = beltrami_coefficient(uv, f, [uv_SEM, zeros(length(uv_SEM),1)]); 
% chop the Beltrami coefficients with extreme distortion
mu(abs(mu)>0.95) = mu(abs(mu)>0.95)./abs(mu(abs(mu)>0.95))*0.95; 
mu(isnan(abs(mu))) = 0.95;

if nargin < 3
    lambda = 0.2; % a value between 0 and 1
end
% reconstruct an initial guess with lower angle distortion
uv_reconstruct = linear_beltrami_solver(uv,f,mu*lambda,bd{1},uv(bd{1},:));  

if ~isempty(find(isnan(uv_reconstruct), 1))
    % encounter some numerical issue 
    mu = beltrami_coefficient(uv_SEM, f, [uv, zeros(length(uv),1)]); 
    uv_reconstruct = linear_beltrami_solver(uv_SEM,f,mu*(1-lambda),bd{1},uv(bd{1},:)); 
end
disk = uv_reconstruct;
fprintf('Done.\n');

%% Step 2: Compute an area-preserving map onto an optimized planar domain using optimal mass transport
fprintf('Step 2: Optimization on the plane...\n');
bd = meshboundaries(f);
uv_bdy = disk(bd{1},:);
va_ori = vertex_area(f,v)/3;
omtoption.eps = 1e-3;
omtoption.max_iter = 15; 

% optimize r
options = optimset('Display','iter','MaxIter',15,'TolX',1e-2);
[r_opt,~] = fminbnd(@(r)optimize_conformality(r),0.01,100,options);

function mean_abs_mu = optimize_conformality(r) 
    % prepare the source and target measure for the OMT computation
    va_uv = vertex_area(f,r*disk)/3;
    % normalize the target area such that sum(va) = int_\Omega sigma
    sigma = @(xy) (4./(1+xy(:,1).^2+xy(:,2).^2).^2); % source measure, with the conformal factor of the stereogrpahic projection
    delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r^2*disk(:,1).^2+r^2*disk(:,2).^2).^2)); % target measure

    try
        % run the OMT map for a given r (Nadeem et al., TVCG 2017)
        [pd,~] = discrete_optimal_transport(r*uv_bdy,f,r*disk,sigma,delta,[],omtoption);
        uv_final = compute_centroid(r*uv_bdy,pd);

    catch 
        % if there is error in the OMT map, try a different initial guess
        uv = disk;
        uv_SEM = Diskmap_SEM(f,v);
        mu = beltrami_coefficient(uv_SEM, f, [uv, zeros(length(uv),1)]); 
        uv_reconstruct = linear_beltrami_solver(uv_SEM,f,mu*lambda*2,bd{1},uv(bd{1},:)); 
        va_uv = vertex_area(f,r*uv_reconstruct)/3;
        % normalize the target area such that sum(va) = int_\Omega sigma
        delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r^2*uv_reconstruct(:,1).^2+r^2*uv_reconstruct(:,2).^2).^2));
        try
            % run the OMT map
            [pd,~] = discrete_optimal_transport(r*uv_bdy,f,r*uv_reconstruct,sigma,delta,[],option);
            uv_final = compute_centroid(r*uv_bdy,pd);
        catch 
            uv = disk;
            uv_SEM = Diskmap_SEM(f,v);
            mu = beltrami_coefficient(uv_SEM, f, [uv, zeros(length(uv),1)]); 
            uv_reconstruct = linear_beltrami_solver(uv_SEM,f,mu*lambda*3,bd{1},uv(bd{1},:)); 
            va_uv = vertex_area(f,r*uv_reconstruct)/3;
            % normalize the target area such that sum(va) = int_\Omega sigma
            delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r^2*uv_reconstruct(:,1).^2+r^2*uv_reconstruct(:,2).^2).^2));
            try
                % run the OMT map
                [pd,~] = discrete_optimal_transport(r*uv_bdy,f,r*uv_reconstruct,sigma,delta,[],option);
                uv_final = compute_centroid(r*uv_bdy,pd);
            catch 
                uv_final = r*uv_reconstruct;
            end
        end
    end

    % inverse stereographic projection
%     z = complex(uv_final(:,1), uv_final(:,2));
%     map = [2*real(z)./(1+abs(z).^2), 2*imag(z)./(1+abs(z).^2), -(-1+abs(z).^2)./(1+abs(z).^2)];

    % evaluate angle distortion or conformality
    mu = beltrami_coefficient(uv_final, f, v);
    mean_abs_mu = mean(abs(mu).^2);
    % fprintf('Done.\n');

end

%% Step 3: Use the optimal r and perform inverse stereographic projection
fprintf('Step 3: Map the result onto the adaptive spherical cap domain...\n');
% use the resulting r 
% prepare the source and target measure for the OMT computation
va_uv = vertex_area(f,r_opt*disk)/3;
% normalize the target area such that sum(va) = int_\Omega sigma
sigma = @(xy) (4./(1+xy(:,1).^2+xy(:,2).^2).^2); % source measure, with the conformal factor of the stereogrpahic projection
delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r_opt^2*disk(:,1).^2+r_opt^2*disk(:,2).^2).^2)); % target measure

try
    % run the OMT map
    [pd,~] = discrete_optimal_transport(r_opt*uv_bdy,f,r_opt*disk,sigma,delta,[],omtoption);
    uv_final = compute_centroid(r_opt*uv_bdy,pd);

catch 
    % if there is error in the OMT map, try a different initial guess
    uv = disk;
    uv_SEM = Diskmap_SEM(f,v);
    mu = beltrami_coefficient(uv_SEM, f, [uv, zeros(length(uv),1)]); 
    uv_reconstruct = linear_beltrami_solver(uv_SEM,f,mu*lambda*2,bd{1},uv(bd{1},:)); 
    va_uv = vertex_area(f,r_opt*uv_reconstruct)/3;
    % normalize the target area such that sum(va) = int_\Omega sigma
    delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r_opt^2*uv_reconstruct(:,1).^2+r_opt^2*uv_reconstruct(:,2).^2).^2));
    try
        % run the OMT map
        [pd,~] = discrete_optimal_transport(r_opt*uv_bdy,f,r_opt*uv_reconstruct,sigma,delta,[],option);
        uv_final = compute_centroid(r_opt*uv_bdy,pd);
    catch 
        uv = disk;
        uv_SEM = Diskmap_SEM(f,v);
        mu = beltrami_coefficient(uv_SEM, f, [uv, zeros(length(uv),1)]); 
        uv_reconstruct = linear_beltrami_solver(uv_SEM,f,mu*lambda*3,bd{1},uv(bd{1},:)); 
        va_uv = vertex_area(f,uv_reconstruct)/3;
        % normalize the target area such that sum(va) = int_\Omega sigma
        delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r_opt^2*uv_reconstruct(:,1).^2+r_opt^2*uv_reconstruct(:,2).^2).^2));
        try
            % run the OMT map
            [pd,~] = discrete_optimal_transport(r_opt*uv_bdy,f,r_opt*uv_reconstruct,sigma,delta,[],option);
            uv_final = compute_centroid(r_opt*uv_bdy,pd);
        catch 
            uv_final = r_opt*uv_reconstruct;
        end
    end
end

% inverse stereographic projection
z = complex(uv_final(:,1), uv_final(:,2));
map = [2*real(z)./(1+abs(z).^2), 2*imag(z)./(1+abs(z).^2), -(-1+abs(z).^2)./(1+abs(z).^2)];
z_low_lim = -(-1+r_opt.^2)./(1+r_opt.^2);
fprintf('Done.\n');

end
