function [map,z_low_lim] = adaptive_area_preserving_map_open(v,f)
% Compute an adaptive area-preserving map of a simply-connected open
% surface to a spherical cap with optimized conformality 
%
% Input:
% v: nv x 3 vertex coordinates of a simply-connected open triangle mesh
% f: nf x 3 triangulations of a simply-connected open triangle mesh
% 
% Output:
% map: nv x 3 vertex coordinates of the open adaptive area-preserving parameterization
% z_low_lim: the lower limit of the z-value of the sphere (everything is with z >= z_low_lim)
%
% If you use this code in your own work, please cite the following paper:
% G. P. T. Choi, A. Giri, L. Kumar, 
% "Adaptive area-preserving parameterization of open and closed anatomical surfaces."
% Computers in Biology and Medicine, 148, 105715, 2022.
% 
% Copyright (c) 2021-2022, Gary Pui-Tung Choi
% https://math.mit.edu/~ptchoi

%% Step 1: Initial flattening map
% based on disk conformal map (Choi and Lui, J. Sci. Comput. 2015)
fprintf('Step 1: Initial flattening map...\n');
disk = disk_conformal_map(v,f);
fprintf('Done.\n');

%% Step 2: Compute an area-preserving map onto an optimized planar domain using optimal mass transport
fprintf('Step 2: Optimization on the plane...\n');
bd = meshboundaries(f);
uv_bdy = disk(bd{1},:);
va_ori = vertex_area(f,v)/3;
omtoption.eps = 1e-3;
omtoption.max_iter = 15; 

% optimize r
options = optimset('Display','iter','MaxIter',15,'TolX',1e-2);
[r_opt,~] = fminbnd(@(r)optimize_conformality(r),0.5,5,options);

function mean_abs_mu = optimize_conformality(r) 
    try
        % prepare the source and target measure for the OMT computation
        va_uv = vertex_area(f,r*disk)/3;
        % normalize the target area such that sum(va) = int_\Omega sigma
        sigma = @(xy) (4./(1+xy(:,1).^2+xy(:,2).^2).^2); % source measure, with the conformal factor of the stereogrpahic projection
        delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r^2*disk(:,1).^2+r^2*disk(:,2).^2).^2)); % target measure

        % run the OMT map for a given r (Nadeem et al., TVCG 2017)
        [pd,~] = discrete_optimal_transport(r*uv_bdy,f,r*disk,sigma,delta,[],omtoption);
        uv_final = compute_centroid(r*uv_bdy,pd);

        % inverse stereographic projection
        z = complex(uv_final(:,1), uv_final(:,2));
        map = [2*real(z)./(1+abs(z).^2), 2*imag(z)./(1+abs(z).^2), -(-1+abs(z).^2)./(1+abs(z).^2)];

        % evaluate angle distortion or conformality
        mu = beltrami_coefficient(uv_final, f, v);
        mean_abs_mu = mean(abs(mu).^2);
        % fprintf('Done.\n');
    catch
        mean_abs_mu = 100000; % encounter error, i.e. bad result
    end
end

%% Step 3: Use the optimal r and perform inverse stereographic projection
fprintf('Step 3: Map the result onto the adaptive spherical cap domain...\n');
% use the resulting r 
% prepare the source and target measure for the OMT computation
va_uv = vertex_area(f,r_opt*disk)/3;
% normalize the target area such that sum(va) = int_\Omega sigma
sigma = @(xy) (4./(1+xy(:,1).^2+xy(:,2).^2).^2); % source measure, with the conformal factor of the stereogrpahic projection
delta = va_ori/sum(va_ori)*sum(va_uv.*(4./(1+r_opt^2*disk(:,1).^2+r_opt^2*disk(:,2).^2).^2)); % target measure

% run the OMT map
[pd,~] = discrete_optimal_transport(r_opt*uv_bdy,f,r_opt*disk,sigma,delta,[],omtoption);
uv_final = compute_centroid(r_opt*uv_bdy,pd);
    
% inverse stereographic projection
z = complex(uv_final(:,1), uv_final(:,2));
map = [2*real(z)./(1+abs(z).^2), 2*imag(z)./(1+abs(z).^2), -(-1+abs(z).^2)./(1+abs(z).^2)];
z_low_lim = -(-1+r_opt.^2)./(1+r_opt.^2);
fprintf('Done.\n');

end
