% adaptive_area_preserving_map_closed: Compute an adaptive area-preserving map 
% of a genus-0 closed surface to a spherical cap with optimized conformality 
% 
% If you use this code in your own work, please cite the following paper:
% G. P. T. Choi, A. Giri, L. Kumar, 
% "Adaptive area-preserving parameterization of open and closed anatomical surfaces."
% Computers in Biology and Medicine, 148, 105715, 2022.
% 
% Copyright (c) 2021-2022, Gary Pui-Tung Choi
% https://math.mit.edu/~ptchoi

addpath('data')
addpath(genpath('code'));

%% Lung surface (Fig. 5a)
load('lung_closed.mat');

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-20 5]);

% adaptive parameterization
map = adaptive_area_preserving_map_closed(v,f,0.2,[0.14, 0.06, 0.22]);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-20 5]);

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Human skull surface (Fig. 5b)
load('vol_OAS2_0103_MR2Scan1.mat'); 
v = vol.bnd(2).pos;
f = vol.bnd(2).tri;

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-100 20]); 

% adaptive parameterization
map = adaptive_area_preserving_map_closed(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-85 5]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Human left ventricle surface (Fig. 5c)
load('Patient006_bnd.mat');
v = bnd.pos; f = bnd.tri;

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-100 20]); 

% adaptive parameterization
map = adaptive_area_preserving_map_closed(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-85 5]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Human cortical surface (Fig. 5d)
load('cortical_surface_closed.mat');

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-120 10]); 

% adaptive parameterization
map = adaptive_area_preserving_map_closed(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-120 10]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);

