% adaptive_area_preserving_map_open: Compute an adaptive area-preserving map 
% of a simply-connected open surface to a spherical cap with optimized conformality 
% 
% If you use this code in your own work, please cite the following paper:
% G. P. T. Choi, A. Giri, L. Kumar, 
% "Adaptive area-preserving parameterization of open and closed anatomical surfaces."
% Computers in Biology and Medicine, 148, 105715, 2022.
% 
% Copyright (c) 2021-2022, Gary Pui-Tung Choi
% https://math.mit.edu/~ptchoi

addpath('data')
addpath(genpath('code'));

%% Human scalp surface (Fig. 4a)
load('vol_OAS2_0104_MR2Scan3.mat'); 
v = vol.bnd(3).pos;
f = vol.bnd(3).tri;
[v,f] = truncate_mesh(v,f,30); 

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-125 20]); 

% adaptive parameterization
map = adaptive_area_preserving_map_open(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-125 20]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Human scalp surface (Fig. 4b)
load('vol_OAS2_0104_MR2Scan3.mat'); 
v = vol.bnd(3).pos;
f = vol.bnd(3).tri;
[v,f] = truncate_mesh(v,f,-55);

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-125 20]); 

% adaptive parameterization
map = adaptive_area_preserving_map_open(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-125 20]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Mannalian tooth surface (Fig. 4c)
load('tooth_open.mat'); 

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([-125 20]); 

% adaptive parameterization
map = adaptive_area_preserving_map_open(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-125 20]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);


%% Human face surface (Fig. 4d)
load('human_face_open.mat');

plot_mesh(v,f); 
title('Input surface')
set(gca,'FontSize',16);
view([0 90]); 

% adaptive parameterization
map = adaptive_area_preserving_map_open(v,f);

plot_mesh(map,f); 
title('Adaptive area-preserving parameterization')
set(gca,'FontSize',16);
view([-166 25]); 

% evaluate distortion 
d = angle_distortion(v,f,map);
a = area_distortion(v,f,map);
fprintf('Mean(|a|) = %.2f, Mean(|d|) = %.2f\n', [mean(abs(a)),mean(abs(d))]);

